# Copyright (c) 2016 Samsung Electronics Co., Ltd All Rights Reserved
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Syntax is:
#   * create_testplan $TEST_PLAN $TEST_REPO
#   * run_testplan $TEST_PLAN $ID
#   * create_chroot $ID
#   * delete_chroot $ID
#   * run_in_chroot $ID $CMD
#   * run_smoke_ssh $IP
#   * run_smoke_sdb $ID
#
# Author: Aleksander Mistewicz <a.mistewicz@samsung.com>

export NATIVE_TCT_HOST_PATH="/opt/tct/tizen_native_3.0"

create_testplan() {
    TEST_PLAN="$1"
    TEST_REPO="$2"
    test -n "${TEST_PLAN}" || die "Missing argument: testplan!"
    test -n "${TEST_REPO}" || die "Missing argument: testrepo!"

    cd "${NATIVE_TCT_HOST_PATH}/shell"
    ./tct-plan-generator --output "${TEST_PLAN}" --match "*" --repository "${TEST_REPO}"
    cd "${OLDPWD}"
}

run_testplan() {
    TEST_PLAN="$1"
    ID="$2"
    test -n "${TEST_PLAN}" || die "Missing argument: testplan!"
    test -n "${ID}" || die "Missing argument: id!"

    cd "${NATIVE_TCT_HOST_PATH}/shell"
    yes | ./tct-shell --testplan "${TEST_PLAN}" --deviceid "${ID}"
    cd "${OLDPWD}"
}

create_chroot() {
    ID="$1"
    test -n "${ID}" || die "Missing argument: id"
    test -d "/mnt/${ID}" && die "Snapshot already created"

    sudo btrfs subvolume snapshot /rootfs/\@ "/mnt/${ID}"
    sudo mount --rbind /dev/ "/mnt/${ID}/dev/"
    sudo mount --rbind /proc/ "/mnt/${ID}/proc/"
    sudo mount --rbind /sys/ "/mnt/${ID}/sys/"
    sudo mount --rbind /run/ "/mnt/${ID}/run/"
    sudo mount --rbind /home/ "/mnt/${ID}/home/"
}

delete_chroot() {
    ID="$1"
    test -n "${ID}" || die "Missing argument: id"

    cut -d ' ' -f 2 /proc/mounts | grep "^/mnt/${ID}" | sort -r | sudo xargs umount
    sudo btrfs subvolume delete "/mnt/${ID}"
}

run_in_chroot() {
    ID="$1"
    CMD="$2"
    test -n "${ID}" || die "Missing argument: id"
    test -n "${CMD}" || die "Missing argument: cmd"

    sudo chroot --userspec=tct "/mnt/${ID}" $CMD
}

run_smoke_ssh() {
    IP="$1"
    test -n "${IP}" || die "Missing argument: ip"
    COUNT=1
    while ! timeout 60 sshpass -p 'tizen' ssh -F "${TESTLAB_SCRIPTS}/conf/ssh_config" \
        "root@${IP}" sh "/tmp/smoke.sh" > sysctl.result
    do
        echo "SSH connection to ${IP} failed: retrying" >&2
        if [ ${COUNT} -ge 6 ]
        then
            die "Timeout: ssh connection failed!"
        fi
        COUNT=$((COUNT+1))
    done
}

run_smoke_sdb() {
    ID="$1"
    test -n "${ID}" || die "Missing argument: id"
    sdb -s "${ID}" shell sh "/tmp/smoke.sh" > sysctl.result
    SLEEP=10
    while ! grep -q "FINISH_SUCCESS" sysctl.result
    do
        if [ ${SLEEP} -ge 20 ]
        then
            sleep "$SLEEP"
        fi
        if [ ${SLEEP} -ge 60 ]
        then
            die "Timeout: sdb connection failed!"
        fi
        sdb -s "${ID}" shell sh "/tmp/smoke.sh" > sysctl.result
        SLEEP=$((SLEEP+10))
    done
}

run_avocado_ssh() {
    IP="$1"
    RESULTS="$2"
    MULTIPLEX_FILE="$3"
    test -n "${IP}" || die "Missing argument: ip"
    test -n "${RESULTS}" || die "Missing argument: results"
    test -n "${MULTIPLEX_FILE}" || die "Missing argument: multiplex_file"
    test -n "${TESTLAB_SCRIPTS}" || die "Missing env: testlab_scripts"

    sed -i "s:IP_VALUE:${IP}:" "${MULTIPLEX_FILE}"
    sed -i "s:SSH_CONFIG_VALUE:${TESTLAB_SCRIPTS}/conf/ssh_config:" "${MULTIPLEX_FILE}"

    avocado run "${TESTLAB_SCRIPTS}/run/common_test.py" \
        --mux-yaml "${MULTIPLEX_FILE}" \
        --job-results-dir "${RESULTS}"
}

run_avocado_sdb() {
    SERIAL="$1"
    RESULTS="$2"
    MULTIPLEX_FILE="$3"
    test -n "${SERIAL}" || die "Missing argument: serial"
    test -n "${RESULTS}" || die "Missing argument: results"
    test -n "${MULTIPLEX_FILE}" || die "Missing argument: multiplex_file"

    sed -i "s:SERIAL_VALUE:${SERIAL}:" "${MULTIPLEX_FILE}"

    avocado run "${TESTLAB_SCRIPTS}/run/common_test.py" \
        --mux-yaml "${MULTIPLEX_FILE}" \
        --job-results-dir "${RESULTS}"
}
